/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set et sw=2 ts=2: */
/***************************************************************************
 *            debug.h
 *
 *  Wed Feb 11 11:22:12 CET 2009
 *  Copyright 2009 Bent Bisballe Nyeng
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *  This file is part of Pracro.
 *
 *  Pracro is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Pracro is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Pracro; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */
#ifndef __PRACRO_DEBUG_H__
#define __PRACRO_DEBUG_H__

#include <stdarg.h>

#ifdef HAVE_CONFIG_H
// For USE_EFENCE
#include <config.h>
#ifdef USE_EFENCE
#include <new>
#include <stdlib.h>
#include <efencepp.h>
#include <efence.h>

// Lazy static alocations makes efence freak out.
// Use this to use hardcoded values instead.
// Currently it disables:
// - gethostbyname
// - getsockname
// - getpeername
// - iconv
//#define BYPASS_STATICALLOCATIONS

#endif/*USE_EFENCE*/
#endif/*HAVE_CONFIG*/

void pracro_debug_init(void);
void pracro_debug_parse(const char *fmt);

enum __pracro_debug_class
{
	__pracro_class_fixme,
	__pracro_class_info,
	__pracro_class_warn,
	__pracro_class_err,
	__pracro_class_debug
};

#ifndef __GNUC__
#error "Need gcc for special debug macro expansions. Please define for other compilers."
#endif

#ifdef WITH_DEBUG
int __pracro_debug(const char *func, const int line, enum __pracro_debug_class cl, const char *ch, const char *fmt, ...) __attribute__((format (printf,5,6)));
int __pracro_debug_va(const char *func, const int line, enum __pracro_debug_class cl, const char *ch, const char *fmt, va_list va);

/* gcc preprocessor magic ahead */
#define __PRACRO_DEBUG_PRINT(cl, ch, fmt...)	\
	do { __pracro_debug(__func__, __LINE__, cl, ch, fmt); } while(0)
#define __PRACRO_DEBUG_PRINT_VA(cl, ch, fmt, a)	\
	do { __pracro_debug_va(__func__, __LINE__, cl, ch, fmt, a); } while(0)
#define __PRACRO_DEBUG(cl, ch, fmt...)		__PRACRO_DEBUG_PRINT(__pracro_class##cl, #ch, fmt)
#define __PRACRO_DEBUG_VA(cl, ch, fmt, a)	__PRACRO_DEBUG_PRINT_VA(__pracro_class##cl, #ch, fmt, a)

#define PRACRO_FIXME(ch, fmt...)	__PRACRO_DEBUG(_fixme, ch, fmt)
#define PRACRO_INFO(ch, fmt...)		__PRACRO_DEBUG(_info, ch, fmt)
#define PRACRO_WARN(ch, fmt...)		__PRACRO_DEBUG(_warn, ch, fmt)
#define PRACRO_ERR(ch, fmt...)		__PRACRO_DEBUG(_err, ch, fmt)
#define PRACRO_DEBUG(ch, fmt...)	__PRACRO_DEBUG(_debug, ch, fmt)
#define PRACRO_FIXME_VA(ch, fmt, a)	__PRACRO_DEBUG_VA(_fixme, ch, fmt, a)
#define PRACRO_INFO_VA(ch, fmt, a)	__PRACRO_DEBUG_VA(_info, ch, fmt, a)
#define PRACRO_WARN_VA(ch, fmt, a)	__PRACRO_DEBUG_VA(_warn, ch, fmt, a)
#define PRACRO_ERR_VA(ch, fmt, a)	__PRACRO_DEBUG_VA(_err, ch, fmt, a)
#define PRACRO_DEBUG_VA(ch, fmt, a)	__PRACRO_DEBUG_VA(_debug, ch, fmt, a)

#define PRACRO_INFO_LOG(ch, fmt...)	PRACRO_INFO(ch, fmt)
#define PRACRO_WARN_LOG(ch, fmt...)	PRACRO_WARN(ch, fmt)
#define PRACRO_ERR_LOG(ch, fmt...)	PRACRO_ERR(ch, fmt)
#define PRACRO_INFO_LOG_VA(ch, fmt, a)	PRACRO_INFO_VA(ch, fmt, a)
#define PRACRO_WARN_LOG_VA(ch, fmt, a)	PRACRO_WARN_VA(ch, fmt, a)
#define PRACRO_ERR_LOG_VA(ch, fmt, a)	PRACRO_ERR_VA(ch, fmt, a)

#else

/* If we compile without debug support, we want them all to go away */
#define PRACRO_FIXME(ch, fmt...)
#define PRACRO_INFO(ch, fmt...)
#define PRACRO_WARN(ch, fmt...)
#define PRACRO_ERR(ch, fmt...)
#define PRACRO_DEBUG(ch, fmt...)
#define PRACRO_FIXME_VA(ch, fmt...)
#define PRACRO_INFO_VA(ch, fmt...)
#define PRACRO_WARN_VA(ch, fmt...)
#define PRACRO_ERR_VA(ch, fmt...)
#define PRACRO_DEBUG_VA(ch, fmt...)

int __pracro_log(const char *func, const int line, enum __pracro_debug_class cl, const char *ch, const char *fmt, ...) __attribute__((format (printf,5,6)));
int __pracro_log_va(const char *func, const int line, enum __pracro_debug_class cl, const char *ch, const char *fmt, va_list va);

#define __PRACRO_LOG_PRINT(cl, ch, fmt...)	\
	do { __pracro_log(__func__, __LINE__, cl, ch, fmt); } while(0)
#define __PRACRO_LOG_PRINT_VA(cl, ch, fmt, a)	\
	do { __pracro_log_va(__func__, __LINE__, cl, ch, fmt, a); } while(0)
#define __PRACRO_LOG(cl, ch, fmt...)	__PRACRO_LOG_PRINT(__pracro_class##cl, #ch, fmt)
#define __PRACRO_LOG_VA(cl, ch, fmt, a)	__PRACRO_LOG_PRINT_VA(__pracro_class##cl, #ch, fmt, a)

#define PRACRO_INFO_LOG(ch, fmt...)	__PRACRO_LOG(_info, ch, fmt)
#define PRACRO_WARN_LOG(ch, fmt...)	__PRACRO_LOG(_warn, ch, fmt)
#define PRACRO_ERR_LOG(ch, fmt...)	__PRACRO_LOG(_err, ch, fmt)
#define PRACRO_INFO_LOG_VA(ch, fmt, a)	__PRACRO_LOG_VA(_info, ch, fmt, a)
#define PRACRO_WARN_LOG_VA(ch, fmt, a)	__PRACRO_LOG_VA(_warn, ch, fmt, a)
#define PRACRO_ERR_LOG_VA(ch, fmt, a)	__PRACRO_LOG_VA(_err, ch, fmt, a)

#endif/*WITH_DEBUG*/

#endif/*__PRACRO_DEBUG_H__*/
