/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/***************************************************************************
 *            lua.cc
 *
 *  Thu May 29 16:30:50 CEST 2008
 *  Copyright 2008 Bent Bisballe Nyeng
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *  This file is part of Pracro.
 *
 *  Pracro is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Pracro is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Pracro; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */
#include "luaresume.h"

#include "debug.h"

#include <stdio.h>

#define GLOBAL_POINTER "_pracroGlobalLUAObjectPointerThisShouldBeANameThatIsNotAccidentallyOverwritten"

static int _getValue(lua_State *L)
{
  int n = lua_gettop(L); // number of arguments
  if(n != 1) {
    char errstr[512];
    sprintf(errstr, "Number of args expected 0, got %d", n);
    lua_pushstring(L, errstr);
    lua_error(L);
    return 1;
  }

  std::string name = lua_tostring(L, lua_gettop(L));

  lua_getglobal(L, GLOBAL_POINTER);
  LUAResume *lua = (LUAResume*)lua_touserdata(L, lua_gettop(L));

  if(!lua) {
    lua_pushstring(L, "No LUA pointer!");
    lua_error(L);
    return 1;
  }

  std::string value = lua->getValue(name);
  lua_pushstring(L, value.c_str());

  return 1;
}

LUAResume::LUAResume(Commit &c)
  : commit(c)
{
  L = luaL_newstate();
  if(L == NULL) {
    error("Could not create LUA state.");
    return;
  }

  luaL_openlibs(L);               

  lua_pushlightuserdata(L, this); // Push the pointer to 'this' instance
  lua_setglobal(L, GLOBAL_POINTER); // Assign it to a global lua var.

  lua_register(L, "getValue", _getValue);
}

LUAResume::~LUAResume()
{
  lua_close(L);
}

std::string LUAResume::getValue(std::string name)
{
  if(commit.fields.find(name) == commit.fields.end()) {
    error("LUAResume: No such field '" + name + "'");
    return "";
  }

  return commit.fields[name];
}

std::string LUAResume::run(std::string program)
{
  if(L == NULL) {
    error("LUA state not initialized!");
    return false;
  }

  PRACRO_DEBUG("Running %s\n", program.c_str());

  /*
  lua_pushstring(L, value.toStdString().c_str());
  lua_setglobal(L, "value");

  lua_pushstring(L, name.toStdString().c_str());
  lua_setglobal(L, "name");
  */

  int top = lua_gettop(L);

  if(luaL_loadbuffer(L, program.c_str(), program.size(), "lua resume generator")) {
    error(lua_tostring(L, lua_gettop(L)));
    return false;
  }

  // Run the loaded code
  if(lua_pcall(L, 0, LUA_MULTRET, 0)) {
    error(lua_tostring(L, lua_gettop(L)));
    return false;
  }

  if(top != lua_gettop(L) - 1) {
    error("Program did not return a single value.\n");
    return false;
  }

  if(lua_isstring(L, lua_gettop(L)) == false) {
    error("Program did not return a string value.\n");
    return false;
  }

  std::string res = lua_tostring(L, lua_gettop(L));
  lua_pop(L, 1);

  return res;
}

void LUAResume::error(std::string message)
{
  printf("LUA ERROR: %s\n", message.c_str());
}
