/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set et sw=2 ts=2: */
/***************************************************************************
 *            macrolist.h
 *
 *  Wed Jul 22 10:26:40 CEST 2009
 *  Copyright 2009 Bent Bisballe Nyeng
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *  This file is part of Pracro.
 *
 *  Pracro is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Pracro is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Pracro; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */
#ifndef __PRACRO_MACROLIST_H__
#define __PRACRO_MACROLIST_H__

#include <map>
#include <string>
#include "versionstr.h"

#include "exception.h"

/**
 * The Items contained in the MacroList.
 */
typedef std::map<VersionStr, std::string> MacroListItem;

/**
 * The MacroList class is intended for macro file caching, so that all macros
 * do not need to be parsed on each macro query.
 * It builds a list of macros and versions based on the informations read from
 * the MacroHeaderParser.
 * This means that just because a macro gets into the list doesn't means that it
 * will validate as a correct macro (not even nessecarily correct XML).
 */
class MacroList : public std::map<std::string, MacroListItem > {
public:
  /**
   * Constructor.
   * @param macropath A std::string containing the path in which we should look
   * for xml files.
   */
  MacroList(std::string macropath);
  
  /**
   * Convenience method, to gain the filename of the latest version of a given macro.
   * This method throws an Exception if the macro does not exist in the tree.
   * @param macro A std::string containing the name of the wanted macro.
   * @return A std::string containing the file containing the macro with full path
   * included.
   */
  std::string getLatestVersion(std::string macro) throw(Exception);

private:
  std::string macropath;
};

#endif/*__PRACRO_MACROLIST_H__*/
