/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set et sw=2 ts=2: */
/***************************************************************************
 *            templatelist.cc
 *
 *  Thu Jul 30 08:52:30 CEST 2009
 *  Copyright 2009 Bent Bisballe Nyeng
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *  This file is part of Pracro.
 *
 *  Pracro is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Pracro is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Pracro; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */
#include "templatelist.h"

#include <sys/types.h>
#include <dirent.h>

#include "debug.h"
#include "templateheaderparser.h"

static std::vector<std::string> listdir(std::string path)
{
  std::vector<std::string> files;

  DIR* dir = opendir(path.c_str());
  if(!dir) {
    PRACRO_ERR(dump, "Could not open directory: %s\n", path.c_str());
    return files;
  }

  struct dirent *d;
  while((d = readdir(dir)) != 0) {
    //if(d->d_type == DT_DIR) {
    std::string name = d->d_name;
    if(name.length() >= 4 && name.substr(name.length() - 4) == ".xml")
      files.push_back(name);
    //}
  }
  closedir(dir);

  return files;
}

TemplateList::TemplateList(std::string templatepath)
{
  this->templatepath = templatepath;
  std::vector<std::string> templates = listdir(templatepath);
  std::vector<std::string>::iterator i = templates.begin();
  while(i != templates.end()) {
    TemplateHeaderParser parser(templatepath + "/" + *i);
    try {
      parser.parse();
      Template *templ = parser.getTemplate();
      (*this)[templ->attributes["name"]][VersionStr(templ->attributes["version"])] = *i;
    } catch(Exception &e) {
      PRACRO_WARN(templatelist, "Skipping %s: %s\n", i->c_str(), e.what());
    }

    i++;
  }

  {
    iterator i = begin();
    while(i != end()) {
      TemplateListItem::iterator j = i->second.begin();
      while(j != i->second.end()) {
        PRACRO_DEBUG(templatelist, "%s - v%s file: %s\n",
                     i->first.c_str(),
                     ((std::string)j->first).c_str(),
                     j->second.c_str());
        j++;
      }
      i++;
    }
  }

}

std::string TemplateList::getLatestVersion(std::string templ) throw(Exception)
{
  if(find(templ) == end()) throw Exception("Template ["+templ+"] does not exist");
  TemplateListItem mli = (*this)[templ];
  if(mli.size() == 0) return "";
  PRACRO_DEBUG(templatelist, "Search for %s - found %s v%s\n",
               templ.c_str(),
               (templatepath + "/" + mli.begin()->second).c_str(),
               ((std::string)mli.begin()->first).c_str());
  return templatepath + "/" + mli.begin()->second;
}

#ifdef TEST_TEMPLATELIST

#define TEMPLATEDIR "/home"  // We assume this directory exists and does not contain any xml files!

int main()
{
  // Test sorting
  TemplateList lst(TEMPLATEDIR);

  lst["template1"][VersionStr("1.0")] = "template1-1.0.xml";
  lst["template1"][VersionStr("1.1")] = "template1-1.1.xml";
  lst["template1"][VersionStr("1.1.1")] = "template1-1.1.1.xml";
  lst["template1"][VersionStr("1.2")] = "template1-1.2.xml";
  lst["template2"][VersionStr("1.0")] = "template2.xml";
  lst["template3"][VersionStr("1.0")] = "template3.xml";

  std::vector<std::string> refs;
  refs.push_back("template1-1.2.xml");
  refs.push_back("template1-1.1.1.xml");
  refs.push_back("template1-1.1.xml");
  refs.push_back("template1-1.0.xml");
  refs.push_back("template2.xml");
  refs.push_back("template3.xml");

  TemplateList::iterator i = lst.begin();
  std::vector<std::string>::iterator k = refs.begin();
  while(i != lst.end()) {
    TemplateListItem::iterator j = i->second.begin();
    while(j != i->second.end()) {
      printf("%s - v%s file: %s - should be %s\n",
             i->first.c_str(),
             ((std::string)j->first).c_str(),
             j->second.c_str(),
             k->c_str());
      if(j->second != *k) return 1;
      j++; k++;
    }
    i++;
  }

  // Test lookup of latest versions.
  std::string m1 = lst.getLatestVersion("template1");
  printf("Latest template1: %s\n", m1.c_str());
  if(m1 != TEMPLATEDIR"/template1-1.2.xml") return 1;

  std::string m2 = lst.getLatestVersion("template2");
  printf("Latest template2: %s\n", m2.c_str());
  if(m2 != TEMPLATEDIR"/template2.xml") return 1;

  std::string m3 = lst.getLatestVersion("template3");
  printf("Latest template3: %s\n", m3.c_str());
  if(m3 != TEMPLATEDIR"/template3.xml") return 1;

  // Look for non existing template (this should throw an exception)
  try {
    std::string m4 = lst.getLatestVersion("template4");
  } catch(Exception &e) {
    printf("ERROR: %s\n", e.what());
    goto onandon;
  }
  return 1;
 onandon:

  return 0;
}

#endif/*TEST_TEMPLATELIST*/
