/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set et sw=2 ts=2: */
/***************************************************************************
 *            templatelist.h
 *
 *  Thu Jul 30 08:52:30 CEST 2009
 *  Copyright 2009 Bent Bisballe Nyeng
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *  This file is part of Pracro.
 *
 *  Pracro is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  Pracro is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Pracro; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */
#ifndef __PRACRO_TEMPLATELIST_H__
#define __PRACRO_TEMPLATELIST_H__

#include <map>
#include <string>
#include "versionstr.h"

#include "exception.h"

/**
 * The Items contained in the TemplateList.
 */
typedef std::map<VersionStr, std::string> TemplateListItem;

/**
 * The TemplateList class is intended for template file caching, so that all templates
 * do not need to be parsed on each template query.
 * It builds a list of templates and versions based on the informations read from
 * the TemplateHeaderParser.
 * This means that just because a template gets into the list doesn't means that it
 * will validate as a correct template (not even nessecarily correct XML).
 */
class TemplateList : public std::map<std::string, TemplateListItem > {
public:
  /**
   * Constructor.
   * @param templatepath A std::string containing the path in which we should look
   * for xml files.
   */
  TemplateList(std::string templatepath);
  
  /**
   * Convenience method, to gain the filename of the latest version of a given template.
   * This method throws an Exception if the template does not exist in the tree.
   * @param template A std::string containing the name of the wanted template.
   * @return A std::string containing the file containing the template with full path
   * included.
   */
  std::string getLatestVersion(std::string templ) throw(Exception);

private:
  std::string templatepath;
};

#endif/*__PRACRO_TEMPLATELIST_H__*/
